#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['BuildCheck']

from coopr.pyomo.base.numvalue import *
from coopr.pyomo.base.plugin import ComponentRegistration
from coopr.pyomo.base.sets import _BaseSet
import types
from coopr.pyomo.base import pyomo


class BuildCheck(object):
    """A build check, which executes a rule for all valid indices"""

    """ Constructor
        Arguments:
           name         The name of this check
           index        The index set that defines the distinct parameters.
                          By default, this is None, indicating that there
                          is a single check.
           rule         The rule that is executed for every indice.
    """
    def __init__(self, *args, **kwd):
        tmpname="unknown"
        self.domain=None
        self._model=None
        self._rule=None
        for key in kwd.keys():
          if key == "name":
             tmpname=kwd[key]
          elif key == "rule":
             self.__dict__["_"+key] = kwd[key]
          else:
             raise ValueError, "BuildCheck constructor: unknown keyword - "+key
        self._index_set=None
        if args == ():
           self._index=[None]
           self._ndim=0
        else:
           self._ndim=0
           for arg in args:
             if not isinstance(arg,_BaseSet):
                raise ValueError, "Cannot index parameter `"+self.name+"' with object of type "+str(type(arg))
             self._ndim += arg.dimen
           if len(args) == 1:
              self._index=args[0]
           else:
              self._index=None
              self._index_set=args
        if not type(self._rule) is types.FunctionType:
            raise ValueEror, "BuildCheck must have an 'rule' option specified whose value is a function"

    def pprint(self, ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, "  ",self.name,":",

    def _construct(self, model, data):
        """ Apply the rule to construct values in this set """
        self._model=model
        if pyomo.debug("verbose"):      #pragma:nocover
           print "Constructing Param, name="+self._name+", from data="+`data`
        for val in self._index:
              if isinstance(val,tuple):
                 tmp = list(val)
              elif val is None:
                 tmp = []
              else:
                 tmp = [val]
              tmp.append(model)
              tmp = tuple(tmp)
              res = self._rule(*tmp)
              if not res:
                    raise ValueError, "BuildCheck %r identified error with index %r" % (self.name, tmp[:-1])


ComponentRegistration("BuildCheck", BuildCheck, "This component is used to perform a test during the model construction process.  The action rule is applied to every index value, and if the function returns False an exception is raised.")

